import pandas as pd
from event_trader.config import PRICE_COL, DATE_COL, SYMBOL_COL

class DemoAccount:
    def __init__(self, initial_cash=1000000, buy_commission=0.0003, sell_commission=0.0008):
        """
        初始化模拟账户。
        :param initial_cash: 初始资金
        :param buy_commission: 买入佣金比例
        :param sell_commission: 卖出佣金比例
        """
        self.initial_cash = initial_cash
        self.cash = initial_cash
        self.holdings = {}  # 持有股票及其数量
        self.buy_commission = buy_commission
        self.sell_commission = sell_commission
        self.transactions = []  # 记录交易记录

    def buy(self, data, index, position=1.0):
        """
        根据策略中的数据行记录进行买入操作。
        :param data: 包含日期、收盘价、开盘价、最低价、最高价、成交量等信息的字典
        :param index: 当前交易数据的索引
        :param position: 买入仓位比例，默认为1.0（全仓）
        """
        symbol = data.get(SYMBOL_COL)
        price = data.get(PRICE_COL)
        date = data.get(DATE_COL)
        
        if price <= 0:
            raise ValueError("价格必须为正数")
        
        if self.cash > 0 and 0 < position <= 1:
            max_shares_to_buy = (self.cash * position) // price
            shares_to_buy = (max_shares_to_buy // 100) * 100
            fee = self.buy_commission * price * shares_to_buy
            cost = shares_to_buy * price * (1 + self.buy_commission)
            if cost <= self.cash and shares_to_buy > 0:
                self.holdings[symbol] = self.holdings.get(symbol, 0) + shares_to_buy
                self.cash -= cost
                self.transactions.append({
                    'date': date,
                    'type': 'buy',
                    'symbol': symbol,
                    'price': price,
                    'shares': shares_to_buy,
                    'cash': self.cash,
                    'fee': fee,
                    'index': index
                })

    def sell(self, data, index, position=1.0):
        """
        根据策略中的数据行记录进行卖出操作。
        :param data: 包含日期、收盘价、开盘价、最低价、最高价、成交量等信息的字典
        :param index: 当前交易数据的索引
        :param position: 卖出仓位比例，默认为1.0（全仓）
        """
        symbol = data.get(SYMBOL_COL)
        price = data.get(PRICE_COL)
        date = data.get(DATE_COL)
        
        if price <= 0:
            raise ValueError("价格必须为正数")
        
        if symbol in self.holdings and self.holdings[symbol] > 0 and 0 < position <= 1:
            shares_to_sell = int(self.holdings[symbol] * position)
            fee = self.sell_commission * price * shares_to_sell
            revenue = shares_to_sell * price * (1 - self.sell_commission)
            self.cash += revenue
            self.holdings[symbol] -= shares_to_sell
            self.transactions.append({
                'date': date,
                'type': 'sell',
                'symbol': symbol,
                'price': price,
                'shares': shares_to_sell,
                'cash': self.cash,
                'fee': fee,
                'index': index
            })

    def get_profit(self):
        return (self.cash - self.initial_cash) / self.initial_cash * 100

    def get_transactions(self):
        return pd.DataFrame(self.transactions)

    def reset_account(self):
        """
        重置账户到初始状态。
        """
        self.cash = self.initial_cash
        self.holdings.clear()
        self.transactions.clear()

    def get_account_status(self):
        """
        获取当前账户状态。
        :return: 账户现金、持股和总资产
        """
        total_assets = self.cash
        for symbol, shares in self.holdings.items():
            if shares > 0:
                # 假设使用最后一笔交易价格计算当前持股市值
                last_price = [t['price'] for t in self.transactions if t['symbol'] == symbol][-1]
                total_assets += shares * last_price
        return {
            'cash': self.cash,
            'holdings': self.holdings,
            'total_assets': total_assets
        }



from .base_strategy import BaseStrategy
from .ma2_strategy import MA2Strategy
from .kdj_strategy import KDJStrategy
from .ma1_strategy import MA1Strategy
from .boll_strategy import BollStrategy
from .macd_strategy import MACDStrategy
from .vma_strategy import VMAStrategy
from .pd_strategy import PriceDeviationStrategy

STRATEGIES = [
    MA2Strategy,
    KDJStrategy,
    MA1Strategy,
    BollStrategy,
    MACDStrategy,
    VMAStrategy,
    PriceDeviationStrategy
]




from event_trader.strategies import BaseStrategy, STRATEGIES
from china_stock_data import StockData
from event_trader.utils import get_first_line
import pandas as pd

class StockInfo:
    def __init__(self, symbol: str, stock_kwargs = {}, strategies = None):
        self.symbol = symbol
        self.stock_data = StockData(symbol, **stock_kwargs)
        self.strategies: dict[str, BaseStrategy] = {}
        strategies = strategies if strategies is not None else STRATEGIES
        for strategy_class in strategies:
            self.strategies[strategy_class.name] = strategy_class(self.stock_data)
            
    def __getattr__(self, key: str):
        if key in self.strategies:
            return self.strategies[key]
        raise AttributeError(f"'StockInfo' object has no attribute '{key}'")
    
    
    def show(self, **kwargs):
        for key, item in self.strategies.items():
            item.show(**kwargs)
            
    def optmize(self):
        for key, item in self.strategies.items():
                item.optimize_parameters()
            
    def get_result(self, optimize = False, strategy = None, opt_params = {},  **kwargs):
        arr = []
        for key, item in self.strategies.items():
            if strategy is not None and key != strategy:
                continue
            
            if optimize:
                item.optimize_parameters(**opt_params)
            account = item.calculate()
            
            arr.append({
                "name": key,
                "description": get_first_line(item.__doc__),
                "parameters": item.parameters,
                'status': item.status(),
                'stock_profit': self.stock_data['涨跌幅'],
                'profit': account.get_profit(),
                'factors': item.factors_value(),
            })
        return pd.DataFrame(arr)
    
    def get_status(self, strategy = None, opt_params = {},  **kwargs):
        arr = []
        for key, item in self.strategies.items():
            if strategy is not None and key != strategy:
                continue
            
            item.calculate_factors()
            
            arr.append({
                "name": key,
                "description": get_first_line(item.__doc__),
                "parameters": item.parameters,
                'status': item.status(),
                'factors': item.factors_value(),
            })
        return pd.DataFrame(arr)

        
        
   


import pandas as pd
from .base_strategy import BaseStrategy
from china_stock_data import StockData
import mplfinance as mpf
from event_trader.config import PRICE_COL

DEFAULT_PARAMS = {
    'short': 12,
    'long': 26,
    'middle': 9,
}

DEFAULT_PARAMS_RANGE = {
    'short': (5, 30),
    'long': (10, 50),
    'middle': (5, 30),
}

# 新增常量定义
LOOKBACK_PERIOD = 20
BUY_THRESHOLD_CLOSE = 1.05
BUY_THRESHOLD_MEDIUM = 1.1
BUY_THRESHOLD_STRICT = 1.03
SELL_THRESHOLD_CLOSE = 0.95
SELL_THRESHOLD_MEDIUM = 0.9
SELL_THRESHOLD_STRICT = 0.97

class MACDStrategy(BaseStrategy):
    """
    MACD 移动平均线收敛/发散指标（Moving Average Convergence Divergence）
    """
    name = 'macd'
    def __init__(self, stock_data: StockData, params = None, params_range = None):
        _params = params if params is not None else DEFAULT_PARAMS
        _params_range = params_range if params_range is not None else DEFAULT_PARAMS_RANGE
        super().__init__(stock_data, MACDStrategy.name, _params, _params_range, None, ['DIF', 'DEA', 'MACD'])
        
    def calculate_factors(self):
        data = self.data
        # 使用pandas的向量化操作
        data['EMA_short'] = data[PRICE_COL].ewm(span=self.short, adjust=False).mean()
        data['EMA_long'] = data[PRICE_COL].ewm(span=self.long, adjust=False).mean()
        data['DIF'] = data['EMA_short'] - data['EMA_long']
        data['DEA'] = data['DIF'].ewm(span=self.middle, adjust=False).mean()
        data['MACD'] = 2 * (data['DIF'] - data['DEA'])
        
    def validate_parameter(self, parameters):
        if parameters['short'] >= parameters['long']:
            return False
        return True 
    
    def buy_signal(self, row, i) -> bool:
        if i < LOOKBACK_PERIOD or pd.isna(row['DIF']) or pd.isna(row['DEA']):
            return False
            
        last = self.data.iloc[i-1]
        
        # 计算过去LOOKBACK_PERIOD天的最低价
        low_period = self.data[PRICE_COL].iloc[i-LOOKBACK_PERIOD:i+1].min()
        
        # 金叉（DIF 上穿 DEA）且价格接近LOOKBACK_PERIOD天低点
        if (row['DIF'] > row['DEA'] and last['DIF'] <= last['DEA'] and
            row[PRICE_COL] < low_period * BUY_THRESHOLD_CLOSE):
            return True
            
        # MACD 柱状图由负转正且逐渐变长，同时价格处于低位
        if (row['MACD'] > 0 and last['MACD'] <= 0 and row['MACD'] > last['MACD'] and
            row[PRICE_COL] < low_period * BUY_THRESHOLD_MEDIUM):
            return True
            
        # 底背离（价格创新低但 MACD 低点抬高）
        if i > 1:
            prev = self.data.iloc[i-2]
            if (row[PRICE_COL] < last[PRICE_COL] and row['MACD'] > last['MACD'] and
                row[PRICE_COL] < low_period * BUY_THRESHOLD_STRICT):
                return True
                
        return False

    def sell_signal(self, row, i) -> bool:
        if i < LOOKBACK_PERIOD or pd.isna(row['DIF']) or pd.isna(row['DEA']):
            return False
            
        last = self.data.iloc[i-1]
        
        # 计算过去LOOKBACK_PERIOD天的最高价
        high_period = self.data[PRICE_COL].iloc[i-LOOKBACK_PERIOD:i+1].max()
        
        # 死叉（DIF 下穿 DEA）且价格接近LOOKBACK_PERIOD天高点
        if (row['DIF'] < row['DEA'] and last['DIF'] >= last['DEA'] and
            row[PRICE_COL] > high_period * SELL_THRESHOLD_CLOSE):
            return True
            
        # MACD 柱状图由正转负且逐渐变短，同时价格处于高位
        if (row['MACD'] < 0 and last['MACD'] >= 0 and row['MACD'] < last['MACD'] and
            row[PRICE_COL] > high_period * SELL_THRESHOLD_MEDIUM):
            return True
            
        # 顶背离（价格创新高但 MACD 高点降低）
        if i > 1:
            prev = self.data.iloc[i-2]
            if (row[PRICE_COL] > last[PRICE_COL] and row['MACD'] < last['MACD'] and
                row[PRICE_COL] > high_period * SELL_THRESHOLD_STRICT):
                return True
                
        return False
        
    def get_plots(self, data):
        high = data['最高'].max()
        lower = data['最低'].max()
        ratio = (high - lower) / 100
        return [
            mpf.make_addplot(data['DIF'] * ratio, color='blue', width=1, label='DIF'),
            mpf.make_addplot(data['DEA'] * ratio, color='orange', width=1, label='DEA'),
            mpf.make_addplot(data['MACD'] * ratio, color='red', width=1, label='MACD')
        ]



import pandas as pd
from .base_strategy import BaseStrategy
from china_stock_data import StockData
import mplfinance as mpf
from event_trader.config import PRICE_COL


DEFAULT_PARAMS = {
    'window': 20,
    "std": 2
}

DEFAULT_PARAMS_RANGE = {
    'window': (5, 40),
    "std": (0.5, 3)
}

DEFAULT_PARAMS_STEP = {
    'std': 0.2
}

class BollStrategy(BaseStrategy):
    """
布林带（BOLL）
中轨（中线） ：
通常是一个特定周期的简单移动平均线（SMA），例如20天的移动平均线。
上轨和下轨：
上轨：中轨加上一个设定的倍数的标准差。
下轨：中轨减去同样倍数的标准差。
标准的倍数通常设定为2，这意味着上、下轨分别在平均值的上下两个标准差处。
    """
    name = 'boll'
    def __init__(self, stock_data: StockData, params = None, params_range = None, params_step = None):
        _params = params if params is not None else DEFAULT_PARAMS
        _params_range = params_range if params_range is not None else DEFAULT_PARAMS_RANGE
        _params_step = params_step if params_step is not None else DEFAULT_PARAMS_STEP
        super().__init__(stock_data, BollStrategy.name, _params, _params_range, _params_step, ['moving_avg', 'upper', 'down'])
        
    def calculate_factors(self):
        window = self.parameters['window']
        self.data['moving_avg'] = self.data[PRICE_COL].rolling(window=window).mean()
        # 计算标准差
        self.data['std'] = self.data[PRICE_COL].rolling(window=window).std()

        # 计算布林带上下轨
        self.data['upper'] = self.data['moving_avg'] + (self.data['std'] * self.parameters['std'])
        self.data['down'] = self.data['moving_avg'] - (self.data['std'] * self.parameters['std'])
        
    def buy_signal(self, row, i) -> bool:
        # 确保布林带数据有效
        if i == 0 or pd.isna(row['down']):
            return False
            
        last = self.data.iloc[i-1]
        
        # 价格触及下轨且有反转迹象
        if row[PRICE_COL] <= row['down'] and row[PRICE_COL] > last[PRICE_COL]:
            return True
            
        # 布林带收缩后的向上突破
        if i > 1:
            prev_band_width = self.data.iloc[i-1]['upper'] - self.data.iloc[i-1]['down']
            curr_band_width = row['upper'] - row['down']
            if curr_band_width < prev_band_width * 0.8:  # 布林带收缩
                if row[PRICE_COL] > row['moving_avg'] and row[PRICE_COL] > last[PRICE_COL]:
                    return True
                    
        # 布林带扩张时的趋势延续
        if i > 1:
            prev_band_width = self.data.iloc[i-1]['upper'] - self.data.iloc[i-1]['down']
            curr_band_width = row['upper'] - row['down']
            if curr_band_width > prev_band_width * 1.2:  # 布林带扩张
                if row[PRICE_COL] > row['moving_avg'] and row[PRICE_COL] > last[PRICE_COL]:
                    return True
                    
        return False

    def sell_signal(self, row, i) -> bool:
        if i==0 or pd.isna(row['upper']):
            return False
            
        last = self.data.iloc[i-1]
        
        # 价格触及上轨且有反转迹象
        if row[PRICE_COL] >= row['upper'] and row[PRICE_COL] < last[PRICE_COL]:
            return True
            
        # 布林带收缩后的向下突破
        if i > 1:
            prev_band_width = self.data.iloc[i-1]['upper'] - self.data.iloc[i-1]['down']
            curr_band_width = row['upper'] - row['down']
            if curr_band_width < prev_band_width * 0.8:  # 布林带收缩
                if row[PRICE_COL] < row['moving_avg'] and row[PRICE_COL] < last[PRICE_COL]:
                    return True
                    
        # 布林带扩张时的趋势延续
        if i > 1:
            prev_band_width = self.data.iloc[i-1]['upper'] - self.data.iloc[i-1]['down']
            curr_band_width = row['upper'] - row['down']
            if curr_band_width > prev_band_width * 1.2:  # 布林带扩张
                if row[PRICE_COL] < row['moving_avg'] and row[PRICE_COL] < last[PRICE_COL]:
                    return True
                    
        return False

    def get_plots(self, data):
        return [
            mpf.make_addplot(data['moving_avg'], width=0.8, color='blue', label=f'{self.parameters["window"]}-Day MA'),
            mpf.make_addplot(data['upper'], width=0.8, color='purple', label=f'{self.parameters["window"]}-Upper'),
            mpf.make_addplot(data['down'], width=0.8, color='green', label=f'{self.parameters["window"]}-Down')
        ]



import pandas as pd
from .base_strategy import BaseStrategy
from china_stock_data import StockData
import mplfinance as mpf
from event_trader.config import PRICE_COL
from event_trader.utils import plot_line_chart
DEFAULT_PARAMS = {
    'short_window': 5,
    'long_window': 20,
}

DEFAULT_PARAMS_RANGE = {
    'short_window': (3, 12),
    'long_window': (12, 40),
}

DEFAULT_PARAMS_STEP = {
    'short_window': 2,
    'long_window': 2,
}

class MA2Strategy(BaseStrategy):
    """
移动平均价格：计算两个不同周期的移动平均值。
当短周期的移动平均线高于长周期的移动平均线时，买入；
当短周期的移动平均线低于长周期的移动平均线时，卖出。
    """
    name = 'ma2'
    def __init__(self, stock_data: StockData, *, sub_path = None, params = None, params_range=None, params_step=None, factors=None):
        if factors is None:
            factors = ['short_mavg', 'long_mavg']
        if sub_path is None:
            sub_path = MA2Strategy.name
        _params = params if params is not None else DEFAULT_PARAMS
        _params_range = params_range if params_range is not None else DEFAULT_PARAMS_RANGE
        _params_step = params_step if params_step is not None else DEFAULT_PARAMS_STEP
        super().__init__(stock_data, sub_path, _params, _params_range, _params_step, factors)

    def calculate_factors(self):
        short_window = self.parameters['short_window']
        long_window = self.parameters['long_window']
        
        # 计算短期和长期移动平均
        self.data['short_mavg'] = self.data[PRICE_COL].rolling(window=short_window).mean()
        self.data['long_mavg'] = self.data[PRICE_COL].rolling(window=long_window).mean()
        
       # 使用单独的变量来存储列，然后进行填充
        short_mavg_derivative = self.data['short_mavg'].diff()
        self.data['short_mavg_derivative'] = short_mavg_derivative.fillna(0)

        long_mavg_derivative = self.data['long_mavg'].diff()
        self.data['long_mavg_derivative'] = long_mavg_derivative.fillna(0)

        
    def validate_parameter(self, parameters):
        if parameters['short_window'] >= parameters['long_window']:
            return False
        return True
    
    def buy_signal(self, row, i) -> bool:
        if i < 2 or pd.isna(row['short_mavg']) or pd.isna(row['long_mavg']):
            return False
        last1 = self.data.iloc[i-1]
        last2 = self.data.iloc[i-2]
        return (row['short_mavg'] < row['long_mavg'] and
                row['long_mavg_derivative'] > 0 and
                row['short_mavg_derivative'] > 0 and
                last1['short_mavg_derivative'] <= 0 and
                last2['short_mavg_derivative'] < 0)

    def sell_signal(self, row, i) -> bool:
        if i < 2 or pd.isna(row['short_mavg']) or pd.isna(row['long_mavg']):
            return False
        last1 = self.data.iloc[i-1]
        last2 = self.data.iloc[i-2]
        return (row['short_mavg'] > row['long_mavg'] and
                row['long_mavg_derivative'] < 0 and
                row['short_mavg_derivative'] < 0 and
                last1['short_mavg_derivative'] >= 0 and
                last2['short_mavg_derivative'] > 0)
        
    def get_plots(self, data):
        return [
            mpf.make_addplot(data['short_mavg'], width=0.8, color='blue', label=f'{self.short_window}-Day MA'),
            mpf.make_addplot(data['long_mavg'], width=0.8, color='orange', label=f'{self.long_window}-Day MA')
        ]
        
    def show_factors(self):
        plot_line_chart(
            self.data, ['short_mavg_derivative', 'long_mavg_derivative'],
            title='Moving avarage derivative',  ylabel= 'derivative')



from event_trader.stock_info import StockInfo
import pandas as pd
from concurrent.futures import ThreadPoolExecutor, as_completed
from .base_stocks import BaseStocks
from .utils import generate_short_md5
import matplotlib.pyplot as plt
import time

def execute_in_threads(iterable, func, max_workers=5):
    results = []
    with ThreadPoolExecutor(max_workers=max_workers) as executor:
        futures = {executor.submit(func, item): item for item in iterable}
        for future in as_completed(futures):
            item = futures[future]
            try:
                result = future.result()
                results.append(result)
            except Exception as e:
                print(f"Error processing {item}: {e}")
    return results

def execute_normal(iterable, func, timespan = 1):
    results = []
    for item in iterable:
        try:
            result = func(item)
            results.append(result)
        except Exception as e:
            print(f"Error processing {item}: {e}")
        time.sleep(timespan)
    return results


class StocksManager(BaseStocks):
    def __init__(self, symbols=None, index=None, start=None, limit=None, **kwargs):
        file_path = generate_short_md5(f'{str(symbols)}-{str(index)}') + '.json'
        super().__init__(symbols=symbols, file_path=file_path, index=index, start=start, limit=limit, **kwargs)
        self.stocks = {}
        self.callbacks = []
        
    def add_callback(self, callback):
        """添加回调函数"""
        self.callbacks.append(callback)

    def get_stock_info(self, symbol, **kwargs):
        if symbol not in self.symbols:
            self.symbols.append(symbol)
        if symbol in self.stocks:
            return self.stocks[symbol]
            
        self.stocks[symbol] = StockInfo(symbol, **kwargs)
        return  self.stocks[symbol]
    
    def show(self, **kwargs):
        def _show(symbol):
            stock = self.get_stock_info(symbol)
            stock.show(**kwargs)
        execute_normal(self.symbols, _show)

    def merge_dataframes(self, dataframes):
        """合并多个 DataFrame"""
        if dataframes:
            return pd.concat(dataframes, ignore_index=True)
        else:
            print("No data to merge.")
            return pd.DataFrame()

    def get_result(self, **kwargs):
        dataframes = []
        def _get_result(symbol):
            stock = StockInfo(symbol)
            df = stock.get_result(**kwargs)
            df['symbol'] = symbol
            
            for callback in self.callbacks:
                callback(df, symbol, self)
                
            return df

        results = execute_normal(self.symbols, _get_result)
        # 处理结果
        for df in results:
            if df is not None:
                dataframes.append(df)

        # 使用公共方法合并 DataFrame
        result_df = self.merge_dataframes(dataframes)
        self.result = result_df
        return result_df

    def optimize(self):
        def _optimize(symbol):
            stock = self.get_stock_info(symbol)
            stock.optmize()
        execute_normal(self.symbols, _optimize)
        
    def __getitem__(self, symbol):
        return self.get_stock_info(symbol)
    
    def show_result(self, count = 2, status = 'Buy', **kwargs):
        if (not hasattr(self, 'result')):
            self.get_result(**kwargs)
        data = self.result
        buy = data[data['status'] == status]
        
        return buy.groupby('symbol').filter(lambda x: len(x) >= count)
            
    def evaluate_strategy_profits(self, **kwargs):
        """评估各策略的平均盈利并绘制图表"""
        if not hasattr(self, 'result'):
            self.get_result(**kwargs)
            
        # 按策略名称分组计算平均盈利
        strategy_profits = self.result.groupby('name')['profit'].mean()
        
        # 绘制柱状图
        plt.figure(figsize=(10, 6))
        strategy_profits.sort_values(ascending=False).plot(kind='bar')
        plt.title('Average Profit by Strategy')
        plt.xlabel('Strategy')
        plt.ylabel('Average Profit')
        plt.xticks(rotation=45)
        plt.tight_layout()
        plt.show()



import pandas as pd
from .base_strategy import BaseStrategy
from china_stock_data import StockData
import mplfinance as mpf
from event_trader.config import PRICE_COL


DEFAULT_PARAMS = {
    'window': 5,
    'percent': 10
}

DEFAULT_PARAMS_RANGE = {
    'window': (3, 35),
    'percent': (2, 20)
}

class PriceDeviationStrategy(BaseStrategy):
    """
股价偏移过大后会回归。
股价降到移动平均价格下方的一定比例后触发回归，是买入的时机；
估计升到了移动平均价格上方一定比例后触发下调，是卖出的时机；
    """
    name = 'pd'
    def __init__(self, stock_data: StockData, params = None, params_range = None):
        _params = params if params is not None else DEFAULT_PARAMS
        _params_range = params_range if params_range is not None else DEFAULT_PARAMS_RANGE
        super().__init__(stock_data, PriceDeviationStrategy.name, _params, _params_range, None, ['moving_avg'])
        
    def calculate_factors(self):
        self.data['moving_avg'] = self.data[PRICE_COL].rolling(window=self.window).mean()
        self.data['percent'] = (self.data[PRICE_COL] - self.data['moving_avg']) * 100 / self.data['moving_avg']
        
    def buy_signal(self, row, i) -> bool:
        if i < self.window + 2:
            return False
        return row['percent'] <=  -self.percent

    def sell_signal(self, row, i) -> bool:
        if i < self.window + 2:
            return False
        return row['percent'] >=  self.percent
    
    def get_plots(self, data):
        return [
            mpf.make_addplot(data['moving_avg'], width=0.8, color='blue', label=f'{self.parameters["window"]}-Day MA')
        ]   



import pandas as pd
import numpy as np
from .base_strategy import BaseStrategy
from china_stock_data import StockData
import mplfinance as mpf
from event_trader.config import PRICE_COL
from event_trader.utils import is_continuous_growth

DEFAULT_PARAMS = {
    'window': 10
}

DEFAULT_PARAMS_RANGE = {
    'window': (5, 50)
}

# 常量定义
RSI_PERIOD = 14
VOLUME_THRESHOLD = 2.5
PRICE_CHANGE_THRESHOLD = 0.03
RSI_OVERSOLD = 30
RSI_OVERBOUGHT = 70

class VMAStrategy(BaseStrategy):
    """
成交量或者换手率指标。
成交量大于移动平均线的2倍;
当价格低于移动平均线时，卖出。
    """
    name = 'vma'
    
    def __init__(self, stock_data: StockData, params=None, params_range=None):
        _params = params if params is not None else DEFAULT_PARAMS
        _params_range = params_range if params_range is not None else DEFAULT_PARAMS_RANGE
        super().__init__(stock_data, VMAStrategy.name, _params, _params_range, None, ['volume_ma', 'price_ma', 'rsi'])
        
    def calculate_factors(self):
        window = self.parameters['window']
        self.data['volume_ma'] = self.data['成交量'].rolling(window=window).mean()
        self.data['price_ma'] = self.data[PRICE_COL].rolling(window=window).mean()
        self.calculate_rsi()
        
    def calculate_rsi(self):
        delta = self.data[PRICE_COL].diff()
        gain = (delta.where(delta > 0, 0)).rolling(window=RSI_PERIOD).mean()
        loss = (-delta.where(delta < 0, 0)).rolling(window=RSI_PERIOD).mean()
        rs = gain / loss
        self.data['rsi'] = 100 - (100 / (1 + rs))
        
    def buy_signal(self, row, i) -> bool:
        if i < self.parameters['window'] or pd.isna(row['volume_ma']) or pd.isna(row['rsi']):
            return False
            
        # 成交量突破且价格处于上升趋势
        volume_breakout = row['成交量'] > VOLUME_THRESHOLD * row['volume_ma']
        price_uptrend = row[PRICE_COL] > row['price_ma'] * (1 + PRICE_CHANGE_THRESHOLD)
        
        # RSI超卖
        rsi_oversold_condition = row['rsi'] < RSI_OVERSOLD
        
        # 连续3天价格上涨
        price_continuous_up = is_continuous_growth(self.data[PRICE_COL].iloc[i-3:i+1], n=3)
        
        return volume_breakout and price_uptrend and rsi_oversold_condition and price_continuous_up


    def sell_signal(self, row, i) -> bool:
        if i < self.parameters['window'] or pd.isna(row['volume_ma']) or pd.isna(row['rsi']):
            return False
            
        # 成交量突破且价格处于下降趋势
        volume_breakout = row['成交量'] > VOLUME_THRESHOLD * row['volume_ma']
        price_downtrend = row[PRICE_COL] < row['price_ma'] * (1 - PRICE_CHANGE_THRESHOLD)
        
        # RSI超买
        rsi_overbought_condition = row['rsi'] > RSI_OVERBOUGHT
        
        # 连续3天价格下跌
        price_continuous_down = price_continuous_down = is_continuous_growth(self.data[PRICE_COL].iloc[i-3:i+1], n=3, reverse=True)
        
        return volume_breakout and price_downtrend and rsi_overbought_condition and price_continuous_down

    def get_plots(self, data):
        return [
            mpf.make_addplot(data['volume_ma'], width=0.8, color='blue', label=f'{self.parameters["window"]}-Day Volume MA'),
            mpf.make_addplot(data['price_ma'], width=0.8, color='red', label=f'{self.parameters["window"]}-Day Price MA'),
            mpf.make_addplot(data['rsi'], panel=1, color='purple', label='RSI')
        ]
